/*
 *  GUI/EntryDialog.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.Reflection;
using Gdk;
using Gtk;
using Glade;
using Mono.Unix;
using Incollector.Engine;
using Incollector.GUI.Widgets;
using Incollector.DataTypes;


namespace Incollector.GUI {
	
	/*
	 * EntryDialog class
	 */
	
	public class EntryDialog {
		protected Glade.XML     gui;
		public    Gtk.Dialog    dialog;
		protected Engine.Entry  EditingEntry;
		protected string        CurrentDataType, DataType, Content;
		protected bool          NewEntry;
		
		
		protected enum EntryTypeEnum {
			Image = 0,
			Title = 1
		}
		
		
		[Glade.Widget] private Gtk.Viewport  Bg, HeadBg;
		[Glade.Widget] public  Gtk.Label     OkBtnLbl, DataTypeName, TagsLabel;
		[Glade.Widget] public  Gtk.Image     OkBtnImg, DataTypeImage;
		[Glade.Widget] private Gtk.VBox      Page;
		[Glade.Widget] private Gtk.HBox      EntryTypeBox;
		[Glade.Widget] private Gtk.ComboBox  EntryType, Rating;
		[Glade.Widget] private Gtk.Table     TagsTable;
		[Glade.Widget] private Gtk.TextView  Comment;
		[Glade.Widget] private Gtk.Button    OkBtn;
		[Glade.Widget] private TagsEntry     Tags;
		[Glade.Widget] protected HSeparator  HeaderSeparator;
		
		
		public EntryDialog (Gtk.Window parent, string [] Tags)
		{
			CreateEntryDialog (parent, Tags, new Engine.Entry (), "", "");
		}
		
		
		public EntryDialog (Gtk.Window parent, string [] Tags, Engine.Entry entry)
		{
			CreateEntryDialog (parent, Tags, entry, "", "");
		}
		
		
		public EntryDialog (Gtk.Window parent, string [] Tags, string DataType, string Content)
		{
			CreateEntryDialog (parent, Tags, new Engine.Entry (), DataType, Content);
		}
		
		
		public EntryDialog (Gtk.Window parent, string [] Tags, Engine.Entry entry, string DataType, string Content)
		{
			CreateEntryDialog (parent, Tags, entry, DataType, Content);
		}
		
		
		private void CreateEntryDialog (Gtk.Window parent, string [] tags, Engine.Entry entry, string DataType, string Content)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "EntryDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("EntryDialog");
			dialog.TransientFor = parent;
			dialog.BorderWidth = 0;
			dialog.VBox.BorderWidth = 0;
			(gui.GetWidget ("dialog-vbox1") as Gtk.VBox).BorderWidth = 0;
			
			Bg.ModifyBg (Gtk.StateType.Normal, parent.Style.Backgrounds [4]);
			
			Tags = new TagsEntry (tags);
			this.TagsLabel.MnemonicWidget = Tags;
			TagsTable.Attach (Tags, 0, 1, 0, 1);
			
			Icons.AssignWindow (dialog);
			
			
			// Rating
			Gtk.ListStore RatingStore = new Gtk.ListStore (typeof (Gdk.Pixbuf));
			Rating.Model = RatingStore;
			
			Gtk.CellRendererPixbuf RPixRend = new Gtk.CellRendererPixbuf ();
			RPixRend.Xpad = 6;
			
			Rating.Clear ();
			Rating.PackStart (RPixRend, false);
			Rating.AddAttribute (RPixRend, "pixbuf", 0);
			
			RatingStore.AppendValues (new object [] {new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "stars.bigger-0.png")});
			RatingStore.AppendValues (new object [] {new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "stars.bigger-1.png")});
			RatingStore.AppendValues (new object [] {new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "stars.bigger-2.png")});
			RatingStore.AppendValues (new object [] {new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "stars.bigger-3.png")});
			
			
			this.EditingEntry = entry;
			this.DataType = DataType;
			
			if (entry.Data != null) {
				this.DataType = EditingEntry.Data.Name;
				
				Comment.Buffer.Text = EditingEntry.Comment;
				Rating.Active       = EditingEntry.Rating;
				Tags.Text           = EditingEntry.GetTagsString (false);
			} else {
				EditingEntry.Date = DateTime.Today;
				Rating.Active = 0;
			}
			
			this.CurrentDataType = "";
			this.Content = Content;
			NewEntry = entry.Data == null;
			
			
			// Entry Type
			Gtk.ListStore store = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string));
			EntryType.Model = store;
			
			Gtk.CellRendererPixbuf PixRend = new Gtk.CellRendererPixbuf ();
			Gtk.CellRendererText TxtRend = new Gtk.CellRendererText ();
			
			EntryType.Clear ();
			EntryType.PackStart (PixRend, false);
			EntryType.AddAttribute (PixRend, "pixbuf", (int) EntryTypeEnum.Image);
			EntryType.PackStart (TxtRend, false);
			EntryType.AddAttribute (TxtRend, "text", (int) EntryTypeEnum.Title);
			
			int i = 0;
			foreach (DataTypes.DataType dtp in DataTypes.DataTypes.DataTypesList) {
				store.AppendValues (new object [] {dtp.Icon32, dtp.Title});
				
				if (String.Compare (dtp.Name, this.DataType) == 0) {
					if (DataType != "")
						this.EditingEntry.Data = DataTypes.DataTypes.GetDataType (dtp.Name).New ();
					EntryType.Active = i;
				}
				
				i++;
			}
			
			
			if ((NewEntry == true) && (DataType == ""))
				try {
					EntryType.Active = Convert.ToInt32 (Engine.Settings.Get ("EntryTypeID"));
				} catch {
					EntryType.Active = 0;
				}
			
			
			if (NewEntry == false) {
				OkBtnLbl.TextWithMnemonic = Catalog.GetString ("_OK");
				OkBtnImg.Stock = Gtk.Stock.Ok;
				dialog.Title = Catalog.GetString ("Editing entry");
				
				EntryTypeBox.Hide ();
				HeaderSeparator.Show ();
				HeadBg.Show ();
				
				Colors.ColorHead (gui);
			}
			
			OkBtn.Sensitive = !NewEntry;
			
			if (EditingEntry.Data != null)
				dialog.Focus = EditingEntry.Data.DefaultFocusWidget;
		}
		
		
		public Engine.Entry GetEntry ()
		{
			EditingEntry.Comment = Comment.Buffer.Text;
			EditingEntry.Rating  = Rating.Active;
			EditingEntry.Data.SynchronizeData ();
			
			EditingEntry.RemoveAllTags ();
			foreach (string tag in Tags.Text.Split (','))
				if (tag.Trim () != "")
					EditingEntry.AddTag (tag.Trim ());
			
			return EditingEntry;
		}
		
		
		public void on_EntryType_changed (object o, EventArgs args)
		{
			if (CurrentDataType != "") {
				int winW, winH;
				dialog.GdkWindow.GetSize (out winW, out winH);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Width", winW);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Height", winH);
			}


			string PreviousDataType = CurrentDataType;
			CurrentDataType = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).Name;
			
			if ((! ((PreviousDataType == "") && (string.Compare (CurrentDataType, DataType) == 0))) ||
						((Content != "") && (PreviousDataType == "")))
				EditingEntry.Data = DataTypes.DataTypes.DataTypesList [EntryType.Active].New ();
			
			
			if (string.Compare (CurrentDataType, PreviousDataType) != 0) {
				if (Page.Children.Length > 0)
					Page.Remove (Page.Children [0]);
				
				Page.PackStart (EditingEntry.Data.EditWidget, true, true, 6);
				
				if ((PreviousDataType == "") && (Content != ""))
					EditingEntry.Data.EditWidgetContent = Content;
				
				EditingEntry.Data.AddChangedEvent (new EventHandler (on_EntryChanged));
			}
			
			
			if (NewEntry)
				OkBtn.Sensitive = false;
			
			int width  = Engine.Settings.GetInt ("EntryDialog." + CurrentDataType + ".Width");
			int height = Engine.Settings.GetInt ("EntryDialog." + CurrentDataType + ".Height");
			if ((width != -1) && (height != -1))
				dialog.Resize (width, height);
			else {
				dialog.Resizable = false;
				Tools.Repaint ();
				dialog.Resizable = true;
			}
			
			
			DataTypeImage.Pixbuf = (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).Icon48;
			DataTypeName.Markup = "<span size='x-large' weight='bold'>" + (DataTypes.DataTypes.DataTypesList [EntryType.Active] as DataTypes.DataType).Title + "</span>";
		}
		
		
		public void on_SomeEntry_activate (object o, EventArgs args)
		{
			dialog.Respond (Gtk.ResponseType.Ok);
		}
		
		
		public void on_EntryDialog_response (object o, ResponseArgs args)
		{
			if (args.ResponseId == ResponseType.Ok) {
				Engine.Settings.Set ("EntryTypeID", EntryType.Active.ToString ());
				
				int winW, winH;
				dialog.GdkWindow.GetSize (out winW, out winH);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Width", winW);
				Engine.Settings.SetInt ("EntryDialog." + CurrentDataType + ".Height", winH);
			}
		}
		
		
		protected void on_EntryChanged (object o, EventArgs args)
		{
			OkBtn.Sensitive = EditingEntry.Data.IsFormFilledIn ();
		}
		
		
		protected void on_EntryDialog_destroy (object o, EventArgs args)
		{
			if (Page.Children.Length > 0)
				Page.Remove (Page.Children [0]);
		}
	}
}
