/*
 *  GUI/ExportEntriesDialog.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using Gdk;
using Mono.Unix;
using Gtk;
using Glade;


namespace Incollector.GUI {
	
	/*
	 * ExportEntriesDialog class
	 */
	
	public class ExportingEntriesDialog {
		private Glade.XML  gui;
		public  Gtk.Dialog dialog;

		[Glade.Widget] public  Gtk.ComboBox ExportType;
		[Glade.Widget] private Gtk.Button   OkBtn;
		[Glade.Widget] public  Gtk.Entry    FileName;
		

		public enum ExportTypeEnum {
			SelectedEntries = 0,
			AllEntries      = 1
		}
		
		
		
		/* Constructor */
		public ExportingEntriesDialog (Gtk.Window parent, bool Selected)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ExportingEntriesDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("ExportingEntriesDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;
			
			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.EXPORT_DIALOG, "Icon", gui);
			Icons.AssignImage (Icons.EXPORT_BUTTON, "OkBtnImg", gui);
			
			FileName.Text = Engine.Settings.Get ("ExportPath");
			Colors.ColorHead (gui);
			

			if (Selected)
				ExportType.Active = (int) ExportTypeEnum.SelectedEntries;
			else
				ExportType.Active = (int) ExportTypeEnum.AllEntries;
		}


		/* BrowseBtn clicked */
		protected void on_BrowseBtn_clicked (object o, EventArgs args)
		{
			dialog.Sensitive = false;

			Gtk.FileChooserDialog dlg = new Gtk.FileChooserDialog (
				Catalog.GetString ("Select file"), dialog, FileChooserAction.Save);
			dlg.SetFilename (FileName.Text);
			dlg.AddButton (Stock.Cancel, ResponseType.Cancel);
			dlg.AddButton (Stock.Ok,     ResponseType.Ok);
			
			dlg.DoOverwriteConfirmation = true;

			if (dlg.Run () == (int) ResponseType.Ok)
				FileName.Text = dlg.Filename;
			
			if ((System.IO.Path.GetExtension (FileName.Text) != ".ied") && (FileName.Text != "") && (FileName.Text != Engine.Settings.Get ("ExportPath")))
				FileName.Text += ".ied";

			dlg.Destroy ();
			dialog.Sensitive = true;
		}


		/* FileName entry changed */
		public void on_FileName_changed (object o, EventArgs args)
		{
			OkBtn.Sensitive = FileName.Text != "";
		}


		/* OkBtn clicked */
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			try {
				string path = System.IO.Path.GetDirectoryName (FileName.Text);
				Engine.Settings.Set ("ExportPath", path);
			} catch {
			}
			
			if (System.IO.Path.GetExtension (FileName.Text) != ".ied") 
				FileName.Text += ".ied";
		}
	}
}
