/*
 *  GUI/ImportEntriesDialog.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using Gdk;
using Mono.Unix;
using Gtk;
using Glade;


namespace Incollector.GUI {
	
	/*
	 * ImportEntriesDialog class
	 */
	
	public class ImportingEntriesDialog {
		private Glade.XML  gui;
		public  Gtk.Dialog dialog;

		[Glade.Widget] public  Gtk.CheckButton ImportDates, ImportRating, ImportTags;
		[Glade.Widget] private Gtk.Button      OkBtn, BrowseBtn;
		[Glade.Widget] public  Gtk.Entry       FileName;
		
		
		
		/* Constructor */
		public ImportingEntriesDialog (Gtk.Window parent, string file)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ImportingEntriesDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("ImportingEntriesDialog");
			dialog.TransientFor = parent;
			dialog.VBox.BorderWidth = 0;

			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.IMPORT_DIALOG, "Icon", gui);
			Icons.AssignImage (Icons.IMPORT_BUTTON, "OkBtnImg", gui);
			
			Colors.ColorHead (Engine.Settings.GetBool ("ColorWindowsHeaders"), gui);
			
			ImportDates.Active  = Engine.Settings.GetBool ("ImportDates");
			ImportRating.Active = Engine.Settings.GetBool ("ImporRating");
			ImportTags.Active   = Engine.Settings.GetBool ("ImportTags");
			FileName.Text       = Engine.Settings.Get ("ImportPath");
			
			if (file != "") {
				FileName.Text = file;
				
				BrowseBtn.Hide ();
				FileName.IsEditable = false;
				FileName.WidthRequest += 60;
			}
		}


		/* BrowseBtn clicked */
		protected void on_BrowseBtn_clicked (object o, EventArgs args)
		{
			dialog.Sensitive = false;

			Gtk.FileChooserDialog dlg = new Gtk.FileChooserDialog (
				Catalog.GetString ("Select file"), dialog, FileChooserAction.Open);
			dlg.AddButton (Stock.Cancel, ResponseType.Cancel);
			dlg.AddButton (Stock.Ok,     ResponseType.Ok);
			dlg.SetFilename (FileName.Text);
			
			if (dlg.Run () == (int) ResponseType.Ok)
				FileName.Text = dlg.Filename;
			

			dlg.Destroy ();
			dialog.Sensitive = true;
		}


		/* FileName entry changed */
		protected void on_FileName_changed (object o, EventArgs args)
		{
			OkBtn.Sensitive = FileName.Text != "";
		}
		
		
		/* OkBtn clicked */
		protected void on_OkBtn_clicked (object o, EventArgs args)
		{
			Engine.Settings.SetBool ("ImportDates", ImportDates.Active);
			Engine.Settings.SetBool ("ImporRating", ImportRating.Active);
			Engine.Settings.SetBool ("ImportTags",  ImportTags.Active);
			
			try {
				string path = System.IO.Path.GetDirectoryName (FileName.Text);
				Engine.Settings.Set ("ImportPath", path);
			} catch {
			}
		}
	}
}
