/*
 *  GUI/MainWindow.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



// if beta version
//#define beta
// ---------------



using System;
using System.Reflection;
using System.Collections;
using System.Threading;
using Gdk;
using Gtk;
using GLib;
using Mono.Unix;
using Incollector.Engine;
using Incollector.GUI.Widgets;
using Incollector.Searching;
using Glade;



namespace Incollector.GUI {
	
	/*
	 * MainWindow class
	 */
	
	public class MainWindow {
		
		private enum PagesEnum {
			Main        = 0,
			Message     = 1,
			BetaVersion = 2
		};
		
		
		public EntriesList Entries {
			get { return this.entries; }
		}
		
		public TagsList Tags {
			get { return this.tags; }
		}
		
		
		private Glade.XML   gui;
		protected string    CurrentSearchText = "";
		public  Gtk.Window  window;
		private EntriesList entries;
		private TagsList    tags;
		private bool        Init, IsTrayIcon;
		public  Gtk.Menu    MainMenuIncollector, MainMenuEntries, MainMenuView, MainMenuHelp;
		protected EntryPreview Preview;
		protected Sidebar      LeftSidebar;
		protected SearchFoldersSidebar FoldersSidebar;
		protected SearchFoldersList    FoldersList;
		
		
		[Glade.Widget] protected Gtk.VBox             EntriesVBox, SearchBox, PreviewBox, LeftSidebarBox;
		[Glade.Widget] protected Gtk.HBox             TopBar;
		[Glade.Widget] protected Gtk.Notebook         Pages, EntriesListNotebook;
		[Glade.Widget] protected Gtk.ToolButton       RemoveBtn;
		[Glade.Widget] protected Gtk.MenuToolButton   AddBtn;
		[Glade.Widget] protected Gtk.ToggleToolButton PreviewTggBtn;
		[Glade.Widget] protected Gtk.Label            MessageA, MessageB, EntriesMsg, SearchLabel;
		[Glade.Widget] public    Gtk.MenuBar          MainMenu;
		[Glade.Widget] protected Gtk.MenuItem         IncollectorMenu, EntriesMenu, ViewMenu, HelpMenu;
		[Glade.Widget] protected Gtk.Toolbar          MainToolbar;
		[Glade.Widget] protected Gtk.CheckMenuItem    show_toolbar, show_tags_sidebar, show_preview;
		[Glade.Widget] protected Gtk.VPaned           EntriesVPaned;
		[Glade.Widget] protected Gtk.HPaned           MainPaned;
		
		
		
		/* Constructor */
		public MainWindow ()
		{
			Init = true;
			
			gui = Glade.XML.FromAssembly ("gui.glade", "MainWindow", null);
			gui.Autoconnect (this);
			window = (Gtk.Window) gui.GetWidget ("MainWindow");
			Icons.AssignWindow (window);
			Icons.AssignImage (Icons.LOGO_BIG, "IncStartLogo", gui);
			Icons.AssignMenuItem (Icons.IMPORT_MENU, "import_entries", gui);
			Icons.AssignMenuItem (Icons.EXPORT_MENU, "export_entries", gui);
			Icons.AssignMenuItem (Icons.STATS_MENU, "stats", gui);
			
			
			// Window location and size from settings
			if ((Engine.Settings.GetInt ("MainWindowPositionX") != -1) && (Engine.Settings.GetInt ("MainWindowPositionY") != -1))
				window.Move (Engine.Settings.GetInt ("MainWindowPositionX"), Engine.Settings.GetInt ("MainWindowPositionY"));
			
			if ((Engine.Settings.GetInt ("MainWindowHeight") != -1) &&
				(Engine.Settings.GetInt ("MainWindowWidth") != -1))
				window.SetDefaultSize (Engine.Settings.GetInt ("MainWindowWidth"),
										Engine.Settings.GetInt ("MainWindowHeight"));
			
			if (Engine.Settings.GetBoolDF ("MainWindowMaximized") == true)
				window.Maximize ();
			
			Pages.ShowTabs = false;
			
			ShowWindowMessage (Catalog.GetString ("Please wait"),
							   Catalog.GetString ("Loading database"));
			window.Show ();
			Tools.Repaint ();
			
			
			// menus
			MainMenuIncollector = IncollectorMenu.Submenu as Gtk.Menu;
			MainMenuEntries     = EntriesMenu.Submenu as Gtk.Menu;
			MainMenuView        = ViewMenu.Submenu as Gtk.Menu;
			MainMenuHelp        = HelpMenu.Submenu as Gtk.Menu;
			
			// Preview widget
			Preview = new EntryPreview ();
			PreviewBox.Add (Preview);
			
			// LeftSidebarBox widget
			LeftSidebar = new Sidebar (new Sidebar.CloseBtnClickedHandler (on_CloseSidebarBtn_clicked));
			LeftSidebarBox.Add (LeftSidebar);
			
			// TagsList widget
			tags = new TagsList (this.window);
			LeftSidebar.AddWidget (tags, Catalog.GetString ("Tags"));
			
			
			// SearchFoldersSidebar
			FoldersSidebar = new SearchFoldersSidebar (this.window);
			LeftSidebar.AddWidget (FoldersSidebar, Catalog.GetString ("Search folders"));
			
			
			// Search folders list
			this.FoldersList = new SearchFoldersList (FoldersSidebar);
			this.FoldersList.ManageFoldersActivated += new SearchFoldersList.ManageFoldersActivatedHandler (this.OnFSManageFoldersActivated);
			this.FoldersList.ShowAll ();
			TopBar.PackStart (this.FoldersList, false, false, 0);
			TopBar.PackStart (new Gtk.HBox (), true, true, 0);
			
			// SearchFolders
			SearchFolders.Activated += new SearchFolders.ActivatedHandler (OnSearchFoldersActivated);
			SearchFolders.Load ();
			SearchFolders.GetFolder (Engine.Settings.GetInt ("SearchFolder")).Activate ();
			
			
			// SearchEntry
			SearchEntry SearchQuery = new SearchEntry (this.window);
			this.TopBar.PackEnd (SearchQuery, false, false, 0);
			
			// entries widget
			entries = new ListView (this.window, Preview, tags, SearchQuery);
			entries.SelectionChanged += new EntriesList.SelectionChangedHandler (on_Entries_selection_changed_handler);
			EntriesVBox.Add (entries);
			
			
			// Showing Preview
			show_preview.Active  = Engine.Settings.GetBool ("ShowPreview");
			PreviewTggBtn.Active = Engine.Settings.GetBool ("ShowPreview");
			PreviewBox.Visible   = Engine.Settings.GetBool ("ShowPreview");
			
			if (Engine.Settings.GetInt ("PreviewPosition") != -1)
				EntriesVPaned.Position = Engine.Settings.GetInt ("PreviewPosition");
			
			if (Engine.Settings.GetInt ("MainPanedPosition") != -1)
				MainPaned.Position = Engine.Settings.GetInt ("MainPanedPosition");
			
			// Showing main toolbar
			show_toolbar.Active = Engine.Settings.GetBool ("ShowMainToolbar");
			MainToolbar.Visible = Engine.Settings.GetBool ("ShowMainToolbar");
			
			// Showing Sidebar
			show_tags_sidebar.Active = Engine.Settings.GetBool ("ShowSidebar");
			LeftSidebarBox.Visible = Engine.Settings.GetBool ("ShowSidebar");
			
			// Tray icon
			Incollector.Widgets.GTrayIcon icon = new Incollector.Widgets.GTrayIcon (
														"Incollector", window);
			IsTrayIcon = icon.IsTrayIcon;
			icon.MiniSearchActivated = new EventHandler (TrayIcon_MiniSearch_activated);
			icon.AddEntryActivated   = new EventHandler (TrayIcon_on_AddEntry_activated);
			icon.ExitActivated       = new EventHandler (TrayIcon_on_Exit_activated);
			icon.DragDataReceived    = new Gtk.DragDataReceivedHandler (TrayIcon_on_DragData_received);
			
			
			// Database 
			DB.Load ();
			Init = false;
			
			
			// Auto save - every 10 seconds
			GLib.Timeout.Add (10000, Autosave);
			
			
			// Menu
			Gtk.Menu AddEntryMenu = new Gtk.Menu ();
			foreach (DataTypes.DataType dtp in DataTypes.DataTypes.DataTypesList) {
				Gtk.ImageMenuItem item = new Gtk.ImageMenuItem (dtp.MenuText);
				item.Name = dtp.Name;
				item.Image = new Gtk.Image (dtp.Icon16);
				AddEntryMenu.Add (item);
				item.Activated += new EventHandler (on_AddSpecifiedEntry_activated);
			}
			
			AddEntryMenu.ShowAll ();
			AddBtn.Menu = AddEntryMenu;
			
			HideWindowMessage ();
			window.Focus = entries.FocusWidget;
			
			
			// HelpUsDialog
			int  count = Incollector.Engine.Settings.GetInt ("launch_cnt");
			bool showdlg = Incollector.Engine.Settings.GetBoolDF ("help_us_dialog_showed");
			if (count == -1)
				Incollector.Engine.Settings.SetInt ("launch_cnt", 1);
			else {
				count++;
				Incollector.Engine.Settings.SetInt ("launch_cnt", count);
			}
			
			if ((count > 30) && (showdlg == false)) {
				Incollector.Engine.Settings.SetBool ("help_us_dialog_showed", true);
				
				HelpUsDialog dlg = new HelpUsDialog (this.window);
				if (dlg.Run () == (int) Gtk.ResponseType.Ok)
					Tools.RunWebBrowser (Urls.DonateWebsite);
				
				dlg.Destroy ();
			}
		}
		
		
		protected void on_Entries_selection_changed_handler (int NItems)
		{
			RemoveBtn.Sensitive = NItems > 0;
			entries.LoadPreview ();
		}
		
		
		protected void OnSearchFoldersActivated (SearchFolder folder)
		{
			if (Init == false)
				tags.ShowAll ();
		}
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   widgets callbacks
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void on_preferences_activate (object o, EventArgs args)
		{
			PreferencesDialog dlg = new PreferencesDialog (window);
			SetSensitive (false);

			dlg.dialog.Show ();
			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				entries.ReloadEntries ();

			dlg.dialog.Destroy ();
			SetSensitive (true);
		}
		
		
		protected void on_hide_window_activate (object o, EventArgs args)
		{
			HideMainWindow ();
		}
		
		
		protected void on_exit_activate (object o, EventArgs args)
		{
			ExitApplication ();
		}
		
		
		protected bool Autosave ()
		{
			GLib.Timeout.Add (10000, Autosave);
			
			SaveAllSettings (true);
			FilesPaths.MoveAutosaveFiles ();
			
			return false;
		}
		
		
		public void SaveAllSettings ()
		{
			SaveAllSettings (false);
		}
		
		
		public void SaveAllSettings (bool Autosave)
		{
			Engine.Settings.SetBool ("ShowPreview", PreviewBox.Visible);
			Engine.Settings.SetInt  ("PreviewPosition", EntriesVPaned.Position);
			Engine.Settings.SetInt  ("MainPanedPosition", MainPaned.Position);
			Engine.Settings.SetBool ("ShowMainToolbar", MainToolbar.Visible);
			Engine.Settings.SetBool ("ShowSidebar", LeftSidebarBox.Visible);
			
			int winX, winY;
			window.GetPosition (out winX, out winY);
			Engine.Settings.SetInt ("MainWindowPositionX", winX);
			Engine.Settings.SetInt ("MainWindowPositionY", winY);
			
			int winW, winH;
			window.GdkWindow.GetSize (out winW, out winH);
			Engine.Settings.SetInt ("MainWindowWidth", winW);
			Engine.Settings.SetInt ("MainWindowHeight", winH);
			
			Engine.Settings.SetBool ("MainWindowMaximized", window.GdkWindow.State == Gdk.WindowState.Maximized);
			
			
			string DisabledTags = "";
			foreach (string tag in tags.DisabledTagsList ())
				DisabledTags += Text.PrepareText (tag, false) + ",";
			Engine.Settings.Set ("DisabledTags", DisabledTags);
			
			Engine.Settings.SetInt ("SearchFolder", SearchFolders.ActiveFolder.ID);
			
			Engine.Settings.Save (Autosave);
			
			
			// DB
			Engine.DB.Save (Autosave);
		}
		
		
		public void ExitApplication ()
		{
			ShowWindowMessage (Catalog.GetString ("Please wait"),
								Catalog.GetString ("Saving database"));
			Tools.Repaint ();
			SaveAllSettings ();
			
			Application.Quit ();
		}
		
		
		protected void on_add_activate (object o, EventArgs args)
		{
			entries.AddEntry ();
		}
		
		
		public void on_import_entries_activate (object o, EventArgs args)
		{
			SetSensitive (false);
			
			string FileName = "";
			if (o.GetType () == typeof (string))
				FileName = o as string;
			
			ImportingEntriesDialog dlg = new ImportingEntriesDialog (window, FileName);
			dlg.dialog.Show ();
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				dlg.dialog.Destroy ();

				ShowWindowMessage (Catalog.GetString ("Please wait"),
								Catalog.GetString ("Importing entries"));
				Tools.Repaint ();
				
				
				if (Engine.DB.ImportEntries (dlg.FileName.Text, dlg.ImportDates.Active,
						dlg.ImportRating.Active, dlg.ImportTags.Active) == false) {
					HideWindowMessage ();
					Tools.Repaint ();
					
					MessageDialog msg2 = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Error,
							ButtonsType.Ok, Catalog.GetString ("The file is broken. entries have not been imported."));
					
					msg2.Run ();
					msg2.Destroy ();
					window.Sensitive = true;
					return;
				}
				
				
				entries.ReloadEntries ();
				
				HideWindowMessage ();
				MessageDialog msg3 = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Info,
						ButtonsType.Ok, Catalog.GetString ("Entries have been imported successfully!"));
				msg3.Run ();
				msg3.Destroy ();
				
				window.Sensitive = false;
			} else
				dlg.dialog.Destroy ();
			
			SetSensitive (true);
		}
		
		
		protected void on_export_entries_activate (object o, EventArgs args)
		{
			SetSensitive (false);

			ExportingEntriesDialog dlg = new ExportingEntriesDialog (window,
							entries.NSelectedEntries >= 1);
			dlg.dialog.Show ();
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				dlg.dialog.Destroy ();
				
				ShowWindowMessage (Catalog.GetString ("Please wait"),
								Catalog.GetString ("Exporting entries"));
				Tools.Repaint ();

				if (dlg.ExportType.Active == 0) // if selected entries
					Engine.DB.ExportEntries (entries.GetSelectedEntries (), dlg.FileName.Text);
				else
					Engine.DB.ExportEntries (new ArrayList (), dlg.FileName.Text);
				
				HideWindowMessage ();
				
				Gtk.MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Info,
							ButtonsType.Ok, Catalog.GetString ("Entries have been exported !"));
				msg.Run ();
				msg.Destroy ();
			} else
				dlg.dialog.Destroy ();
			
			SetSensitive (true);
		}
		
		
		protected void on_stats_activate (object o, EventArgs args)
		{
			this.SetSensitive (false);
			
			StatsDialog dlg = new StatsDialog (this.window);
			dlg.Run ();
			dlg.Destroy ();
			
			this.SetSensitive (true);
		}
		
		
		protected void on_show_toolbar_activate (object o, EventArgs args)
		{
			if ((o as CheckMenuItem).Active)
				MainToolbar.Show ();
			else
				MainToolbar.Hide ();
		}
		
		
		protected void on_show_tags_sidebar_activate (object o, EventArgs args)
		{
			if ((o as CheckMenuItem).Active)
				LeftSidebarBox.Show ();
			else
				LeftSidebarBox.Hide ();
		}
		
		
		protected void on_show_preview_activate (object o, EventArgs args)
		{
			PreviewTggBtn.Active = (o as CheckMenuItem).Active;
			PreviewBox.Visible = (o as CheckMenuItem).Active;
		}
		
		
		protected void on_about_activate (object o, EventArgs args)
		{
			SetSensitive (false);
			
			AboutWindow wdw = new AboutWindow (this.window);
			wdw.Run ();
			wdw.Destroy ();
			
			SetSensitive (true);
		}
		
		
		protected void OnFSManageFoldersActivated ()
		{
			Gtk.CheckMenuItem item = (show_tags_sidebar as Gtk.CheckMenuItem);
			
			if (item.Active == false) {
				item.Activate ();
				return;
			}
			
			
			item.Active = false;
			Tools.Repaint ();
			System.Threading.Thread.Sleep (150);
			item.Activate ();
			
			this.LeftSidebar.ActivateWidget (this.FoldersSidebar);
		}
		
		
		/* MainWindow delete */
		protected void on_MainWindow_delete_event (object o, DeleteEventArgs args)
		{
			if ((Engine.Settings.GetBool ("MinimizeToTray") == false) || (IsTrayIcon == false)) {
				ExitApplication ();
				args.RetVal = true;
			} else {
				HideMainWindow ();
				args.RetVal = true;
			}
		}
		
		
		private void HideMainWindow ()
		{
			if (Engine.Settings.GetBool ("CloseHideMsg") == true) {
				TrayMessage msg = new TrayMessage (MsgType.Info, true, null, null,
									Catalog.GetString ("Incollector window has been hidden"),
									Catalog.GetString ("To restore window click Incollector icon in your notification area."));
				msg.Show ();
				Engine.Settings.SetBool ("CloseHideMsg", false);
			}
			
			window.Hide ();
		}
		
		
		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			entries.AddEntry ("", "");
		}
		
		
		protected void on_AddSpecifiedEntry_activated (object o, EventArgs args)
		{
			entries.AddEntry ("", (o as MenuItem).Name);
		}
		
		
		/* RemoveBtn clicked */
		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			entries.RemoveSelectedEntries ();
		}
		
		
		/* PreviewTggBtn clicked */
		protected void on_PreviewTggBtn_toggled (object o, EventArgs args)
		{
			show_preview.Active = PreviewTggBtn.Active;
			if (PreviewTggBtn.Active)
				entries.LoadPreview (true);
		}
		
		
		protected void on_ShowEntries_changed (object o, EventArgs args)
		{
			SetSensitive (false);

			if (Init == false)
				entries.ReloadEntries ();
			
			SetSensitive (true);
		}
		
		// TODO: move it !
		protected void on_CloseSidebarBtn_clicked ()
		{
			show_tags_sidebar.Active = ! show_tags_sidebar.Active;
		}
		


		////////////////////////////////////////////////////////////////////////
		//
		//   window functions
		//
		////////////////////////////////////////////////////////////////////////
		
		
		/// <summary>Sets Main window sensitive</summary>
		public void SetSensitive (bool sensitive)
		{
			window.Sensitive = sensitive;
			while (Application.EventsPending ())
				Application.RunIteration ();
		}
		
		
		/// <summary>Shows window message</summary>
		public void ShowWindowMessage (string title, string text)
		{
			MessageA.Markup = "<span size='xx-large' weight='bold'>" + title + "</span>";
			MessageB.Markup = text;
			
			Pages.Page = (int) PagesEnum.Message;
		}
		
		
		/// <summary>Hides window message</summary>
		public void HideWindowMessage ()
		{
			Pages.Page = (int) PagesEnum.Main;
		}
		
		
		
		////////////////////////////////////////////////////////////////////////
		//
		//   TrayIcon
		//
		////////////////////////////////////////////////////////////////////////
		
		protected void TrayIcon_on_Exit_activated (object o, EventArgs args)
		{
			ExitApplication ();
		}
		
		
		protected void TrayIcon_MiniSearch_activated (object o, EventArgs args)
		{
			MiniSearchWindow wdw = new MiniSearchWindow ();
			wdw.window.Show ();
		}
		
		
		protected void TrayIcon_on_AddEntry_activated (object o, EventArgs args)
		{
			on_add_activate (null, null);
		}
		
		
		protected void TrayIcon_on_DragData_received (object o, DragDataReceivedArgs args)
		{
			string url = new System.Text.StringBuilder (System.Text.Encoding.UTF8.GetString (args.SelectionData.Data)).ToString ();
			
			try {
				Uri addr = new Uri (url);
					
				url = url.Replace ("file:///", "").Trim ();
				
				if (! Engine.FilesPaths.IsWinSystem ())
					url = "/" + url;
				else {
					url = url.Replace (Convert.ToChar (13), '*');
					url = url.Replace (Convert.ToChar (10), '*');
					url = url.Replace ('/', '\\');
					url = url.Replace ("*", "");
				}
			
				if ((addr.IsFile == true) && (System.IO.File.Exists (url.ToString ()))) { // if file
					// checking file extension
					if (System.IO.Path.GetExtension (url) != ".ied") {
						window.Sensitive = false;
						MessageDialog msg = new MessageDialog (this.window, DialogFlags.DestroyWithParent, MessageType.Error,
								ButtonsType.Ok, Catalog.GetString ("The file has an invalid extension."));
						msg.Show ();
						msg.Run ();
						msg.Destroy ();
						window.Sensitive = true;
						return;
					}
					
					on_import_entries_activate (url, null);
				} else
					if ((addr.ToString ().StartsWith ("http")) || (addr.ToString ().StartsWith ("ftp")))
						entries.AddEntry (addr.ToString (), "webaddress");	
			} catch {
				return;
			}
		}
	}
}
