/*
 *  GUI/MiniSearchWindow.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using Gdk;
using Mono.Unix;
using Gtk;
using Glade;
using Incollector.DataTypes;
using Incollector.Engine;
using Incollector.GUI.Widgets;
using System.Collections;


namespace Incollector.GUI {
	
	/*
	 * MiniSearchWindow class
	 */
	
	public class MiniSearchWindow {
	
		protected Glade.XML   gui;
		protected SearchEntry SearchKeyword;
		public    Gtk.Window  window;
		
		
		[Glade.Widget] protected Gtk.TreeView EntriesList;
		[Glade.Widget] protected Gtk.VBox     MainBox;
		[Glade.Widget] protected Gtk.Button   CloseBtn;
		[Glade.Widget] protected Gtk.HBox     SearchHBox, BottomBox;
		[Glade.Widget] protected Gtk.Viewport EntriesListViewport;
		
		
		Gdk.Pixbuf [] Stars = {
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-0.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-1.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-2.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-3.png"))
			};
		
		protected enum ColsEnum {
			Icon   = 0,
			Title  = 1,
			Rating = 2,
			ID     = 3 
		}
		
		
		public MiniSearchWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "MiniSearchWindow", null);
			gui.Autoconnect (this);
			window = (Gtk.Window) gui.GetWidget ("MiniSearchWindow");
			window.KeepAbove = true;
			window.Decorated = false;
			
			this.SearchKeyword = new SearchEntry (this.window);
			this.SearchKeyword.FindActivated += new SearchEntry.FindActivateHandler (OnSearchKeywordFindActivated);
			this.SearchKeyword.Escaped += new SearchEntry.EscapeHandler (OnSearchKeywordEscaped);
			this.SearchHBox.PackStart (this.SearchKeyword, true, true, 0);
			
			
			Gtk.ListStore store = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (Gdk.Pixbuf), typeof (int));
			EntriesList.Model = store;
			EntriesList.EnableSearch = false;
			EntriesList.RulesHint = true;
			EntriesList.HeadersVisible = false;
			EntriesList.KeyReleaseEvent += new Gtk.KeyReleaseEventHandler (on_EntriesList_key_released);
			
			
			CellRendererPixbuf rc0 = new CellRendererPixbuf ();
			rc0.Xpad = 5;
			rc0.Xalign = 0;
			EntriesList.AppendColumn ("", rc0, "pixbuf", 0);
			
			CellRendererText rc1 = new CellRendererText ();
			rc1.Ellipsize = Pango.EllipsizeMode.End;
			TreeViewColumn c1 = new TreeViewColumn ("", rc1, "markup", 1);
			c1.Expand = true;
			EntriesList.AppendColumn (c1);
			
			CellRendererPixbuf rc2 = new CellRendererPixbuf ();
			rc2.Xpad = 5;
			rc2.Xalign = 1;
			EntriesList.AppendColumn ("", rc2, "pixbuf", 2);
			
			
			Colors.Modify (new Gdk.Color (65, 65, 65), this.window);
			Colors.Modify (new Gdk.Color (255, 255, 255), "EntriesListViewport", gui);
			this.SearchKeyword.SetEntryColor (new Gdk.Color (183, 255, 163));
			this.SearchKeyword.SetLabelColor (new Gdk.Color (255, 255, 255));
			Icons.AssignImage (Icons.LOGO_36, "IcIcon", gui);
			
			
			MainBox.Remove (SearchHBox);
			GradientBox box2 = new GradientBox (new Cairo.Color (0.29, 0.29, 0.29, 1.0),
												new Cairo.Color (0.22, 0.22, 0.22, 1.0));
			MainBox.Add (box2);
			box2.PackStart (SearchHBox, true, true, 0);
			box2.Show ();
			
			
			MainBox.Remove (EntriesListViewport);
			MainBox.PackStart (EntriesListViewport);
			
			
			MainBox.Remove (BottomBox);
			GradientBox box1 = new GradientBox (new Cairo.Color (0.70, 0.73, 0.89, 1.0),
												new Cairo.Color (1.0, 1.0, 1.0, 1.0));
			MainBox.Add (box1);
			box1.PackEnd (BottomBox, true, true, 0);
			box1.Show ();
			
			
			LabelButton CloseLB = new LabelButton (Catalog.GetString ("Close this window"), Gtk.ArrowType.Up);
			CloseLB.NormalColor = new Gdk.Color (0, 0, 0);
			CloseLB.HoverColor = new Gdk.Color (70, 70, 70);
			CloseLB.Clicked += new LabelButton.ClickHandler (this.OnCloseLBClicked);
			BottomBox.Add (CloseLB);
			
			
			window.Focus = SearchKeyword.FocusWidget;
		}
		
		
		protected void OnSearchKeywordFindActivated (string Query)
		{
			(EntriesList.Model as ListStore).Clear ();
			
			if (Query == "") {
				EntriesListViewport.Hide ();
				BottomBox.Hide ();
				return;
			}
			
			Hashtable TypeIcons = new Hashtable ();
			foreach (DataType type in DataTypes.DataTypes.DataTypesList)
				TypeIcons [type.Name] = type.Icon16;
			
			foreach (EntryStruct ent in Engine.DB.Data) {
				Engine.Entry entry = new Engine.Entry (ent);
				if ((entry.Data.Search (Query)) || (Engine.Text.Search (Query, entry.Comment)))
					AddItem (entry, TypeIcons [entry.Data.Name] as Gdk.Pixbuf);
			}
			
			EntriesListViewport.Show ();
			BottomBox.Show ();
		}
		
		
		protected void OnSearchKeywordEscaped ()
		{
			this.window.Destroy ();
		}
		
		
		protected void OnCloseLBClicked ()
		{
			this.window.Destroy ();
		}
		
		
		protected void on_EntriesList_key_released (object o, Gtk.KeyReleaseEventArgs args)
		{
			if (args.Event.Key == Gdk.Key.Escape)
				this.SearchKeyword.Escape ();
		}
		
		
		protected void on_EntriesList_row_activated (object o, RowActivatedArgs args)
		{
			if (EntriesList.Selection.CountSelectedRows () != 1)
				return;
			
			Gtk.TreeIter itr;
			Gtk.ListStore store = EntriesList.Model as ListStore;
			store.GetIter (out itr, EntriesList.Selection.GetSelectedRows () [0]);


			EntryDialog dlg = new EntryDialog (this.window, new string [] {}, 
						new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID)));
			dlg.dialog.Show ();
			
			if (dlg.dialog.Run () == (int) ResponseType.Ok) {
				Engine.Entry entry = dlg.GetEntry ();
				entry.Save ();
				
				store.SetValue (itr, (int) ColsEnum.Icon, entry.Data.Icon16);
				store.SetValue (itr, (int) ColsEnum.Title,Engine.Text.PrepareText (Engine.Text.Crop (entry.Data.EntryTitle), true));
				store.SetValue (itr, (int) ColsEnum.Rating, Stars [entry.Rating]);
			}
			
			dlg.dialog.Destroy ();
		}
		
		
		protected void AddItem (Engine.Entry entry, Gdk.Pixbuf icon)
		{
			(EntriesList.Model as ListStore).AppendValues (new object [] {
						icon,
						Engine.Text.PrepareText (Engine.Text.Crop (entry.Data.EntryTitle), true),
						Stars [entry.Rating], entry.ID
					});
		}
	}
}
