/*
 *  GUI/PreferencesDialog.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using Incollector.Engine;
using Glade;
using Gtk;
using Mono.Unix;
using System.Collections;


namespace Incollector.GUI {

	/*
	 * PreferencesDialog class
	 */


	public class PreferencesDialog  {
		
		private Glade.XML  gui;
		public  Gtk.Dialog dialog;

		[Glade.Widget] private Gtk.Button      OkBtn;
		[Glade.Widget] private Gtk.Table       ColorsTable;
		[Glade.Widget] private Gtk.ColorButton SidebarColor, SearchPanelColor,
										ImportedEntriesPanelColor;
		[Glade.Widget] private Gtk.ComboBox    InternetBrowser;
		[Glade.Widget] private Gtk.Label       CommandLabel;
		[Glade.Widget] private Gtk.Entry       Command;
		
		private ArrayList Browsers;



		public PreferencesDialog (Gtk.Window parent)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "PreferencesDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("PreferencesDialog");
			dialog.TransientFor = parent;

			Icons.AssignWindow (dialog);
			Icons.AssignImage (Icons.MAIN_SETTINGS_TAB, "MainSettingsImg", gui);
			Icons.AssignImage (Icons.LOOKNFEEL_TAB, "LNFImg", gui);


			GetBool ("DateFirstYear");
			GetBool ("Time24Mode");
			GetBool ("MinimizeToTray");
			GetBool ("ShowOnlyTrayIcon");
			GetBool ("ColorWindowsHeaders");
			
			
			// loading internet browsers list
			Browsers = new ArrayList ();
			string bwr = Engine.Settings.Get ("webbrowser");
			
			if (Engine.FilesPaths.IsWinSystem () == true) {
				if (Tools.CheckCommand ("iexplore.exe", "") != "") {
					InternetBrowser.AppendText ("Internet Explorer");
					Browsers.Add (Tools.CheckCommand ("iexplore.exe", ""));
					InternetBrowser.Active = Browsers.Count - 1;
				}
			} else {
				string cmd = Tools.CheckCommand ("epiphany", "-n");
				if (cmd != "") {
					InternetBrowser.AppendText ("Epiphany");
					Browsers.Add (cmd);
					
					if (String.Compare (bwr, cmd) == 0)
						InternetBrowser.Active = Browsers.Count - 1;
				}
				
				cmd = Tools.CheckCommand ("firefox", "");
				if (cmd != "") {
					InternetBrowser.AppendText ("Firefox");
					Browsers.Add (cmd);
					
					if (String.Compare (bwr, cmd) == 0)
						InternetBrowser.Active = Browsers.Count - 1;
				}
				
				cmd = Tools.CheckCommand ("konqueror", "");
				if (cmd != "") {
					InternetBrowser.AppendText ("Konqueror");
					Browsers.Add (cmd);
					
					if (String.Compare (bwr, cmd) == 0)
						InternetBrowser.Active = Browsers.Count - 1;
				}
				
				cmd = Tools.CheckCommand ("mozilla", "");
				if (cmd != "") {
					InternetBrowser.AppendText ("Mozilla");
					Browsers.Add (cmd);
					
					if (String.Compare (bwr, cmd) == 0)
						InternetBrowser.Active = Browsers.Count - 1;
				}
				
				cmd = Tools.CheckCommand ("opera", "");
				if (cmd != "") {
					InternetBrowser.AppendText ("Opera");
					Browsers.Add (cmd);
					
					if (String.Compare (bwr, cmd) == 0)
						InternetBrowser.Active = Browsers.Count - 1;
				}
			}
			
			InternetBrowser.AppendText (Catalog.GetString ("Other browser"));
			
			if ((InternetBrowser.Active == -1) && (bwr != "")) {
				Browsers.Add (bwr);
				InternetBrowser.Active = Browsers.Count - 1;
				Command.Text = bwr;
			}
		}
		
		
		protected void on_OkBtn_clicked (object o, EventArgs args)
		{
			// saving settings
			SetBool ("DateFirstYear");
			SetBool ("Time24Mode");
			SetBool ("MinimizeToTray");
			SetBool ("ShowOnlyTrayIcon");
			SetBool ("ColorWindowsHeaders");
			
			
			// Internet browser
			if (InternetBrowser.Active != -1)
				if (InternetBrowser.Active == Browsers.Count)
					Engine.Settings.Set ("webbrowser", Command.Text);
				else
					Engine.Settings.Set ("webbrowser", (string) Browsers [InternetBrowser.Active]);
		}
		
		
		protected void on_InternetBrowser_changed (object o, EventArgs args)
		{
			CommandLabel.Visible = String.Compare (InternetBrowser.ActiveText, Catalog.GetString ("Other browser")) == 0;
			Command.Visible = String.Compare (InternetBrowser.ActiveText, Catalog.GetString ("Other browser")) == 0;
		}


		protected void on_name_changed (object o, EventArgs args)
		{
			OkBtn.Sensitive = (o as Gtk.Entry).Text != "";
		}


		protected void on_UseDefaultColors_toggled (object o, EventArgs args)
		{
			ColorsTable.Sensitive = ! (o as CheckButton).Active;
		}

		
		protected void on_ResetColorSidebar_clicked (object o, EventArgs args)
		{
			SidebarColor.Color     = new Gdk.Color (237, 237, 237);
		}
		
		
		protected void on_ResetColorSearchPanel_clicked (object o, EventArgs args)
		{
			SearchPanelColor.Color = new Gdk.Color (230, 230, 230);
		}
		
		
		protected void on_ResetColorImportedEntriesPanel_clicked (object o, EventArgs args)
		{
			ImportedEntriesPanelColor.Color = new Gdk.Color (230, 230, 230);
		}


		/// <summary>Sets Gtk.CheckButton (named as key) value as value (named as key) from settings</summary>
		private void GetBool (string key)
		{
			(gui.GetWidget (key) as Gtk.CheckButton).Active = Engine.Settings.GetBool (key);
		}


		/// <summary>Sets value (named as key) in settings as Gtk.CheckButton (named as key) value</summary>
		protected void SetBool (string key)
		{
			Engine.Settings.SetBool (key, (gui.GetWidget (key) as Gtk.CheckButton).Active);
		}
		
		
		/// <summary>Sets Gtk.SpinButton (named as key) value as value (named as key) from settings</summary>
		protected void GetInt (string key)
		{
			(gui.GetWidget (key) as Gtk.SpinButton).Value = Engine.Settings.GetInt (key);
		}


		/// <summary>Sets value (named as key) in settings as Gtk.SpinButton (named as key) value</summary>
		protected void SetInt (string key)
		{
			Engine.Settings.SetInt (key, Convert.ToInt32 ((gui.GetWidget (key) as Gtk.SpinButton).Value));
		}


		/// <summary>Sets Gtk.ColorButton (named as key) value as value (named as key) from settings</summary>
		protected void GetColor (string key)
		{
			(gui.GetWidget (key) as Gtk.ColorButton).Color = Engine.Settings.GetColor (key);
		}


		/// <summary>Sets value (named as key) in settings as Gtk.ColorButton (named as key) value</summary>
		protected void SetColor (string key)
		{
			Engine.Settings.SetColor (key, (gui.GetWidget (key) as Gtk.ColorButton).Color);
		}
	}
}
