/*
 *  GUI/Widgets/PreviewLabel.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using Mono.Unix;
using Gtk;
using Gdk;


namespace Incollector.GUI.Widgets {
	
	public class PreviewLabel : Gtk.EventBox {
		
		protected Gtk.Label label;
		
		protected TargetEntry [] TargetTable = new TargetEntry [] {
			new TargetEntry ("STRING", 0, 0)
		};
		
		
		public float Xalign {
			get { return this.label.Xalign;  }
			set { this.label.Xalign = value; }
		}
		
		public float Yalign {
			get { return this.label.Yalign;  }
			set { this.label.Yalign = value; }
		}
		
		public string Text {
			get { return this.label.Text;  }
			set { this.label.Text = value; }
		}
		
		public bool Wrap {
			get { return this.label.Wrap;  }
			set { this.label.Wrap = value; }
		}
		
		public int Width {
			get { return this.label.WidthRequest; }
			set { this.label.WidthRequest = value; }
		}
		
		
		public PreviewLabel (string text)
		{
			this.VisibleWindow = false;
			this.ButtonPressEvent += new ButtonPressEventHandler (on_Clicked);
			this.DragDataGet += new DragDataGetHandler (on_drag_data_get);
			
			Gtk.Drag.SourceSet (this, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
				    TargetTable, DragAction.Copy | DragAction.Move);
			
			this.label = new Gtk.Label ();
			this.label.UseMarkup = true;
			this.label.UseUnderline = false;
			this.label.Selectable = false;
			this.label.Text = text;
			
			this.Add (this.label);
		}
		
		
		protected void on_drag_data_get (object o, DragDataGetArgs args)
		{
			byte [] data = System.Text.Encoding.UTF8.GetBytes (this.Text);
			Gdk.Atom [] targets = args.Context.Targets;
			
			args.SelectionData.Set (targets[0], 8, data, data.Length);
		}
		
		
		protected void on_Clicked (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3) {
				Menu menu = new Menu ();

				Gtk.ImageMenuItem CopyLink = new Gtk.ImageMenuItem (Catalog.GetString ("Copy"));
				CopyLink.Image = new Gtk.Image (Stock.Copy, Gtk.IconSize.Menu);
				CopyLink.Activated += new EventHandler (on_CopyLink_activate);
				menu.Add (CopyLink);
				
				menu.ShowAll ();
				menu.Popup ();
			}
		}
		
		
		protected void on_CopyLink_activate (object o, EventArgs args)
		{
			Clipboard.Get (Gdk.Atom.Intern ("CLIPBOARD", true)).Text = this.Text;
		}
		
		
		public new void ModifyFont (Pango.FontDescription desc)
		{
			this.label.ModifyFont (desc);
		}
	}
}