/*
 *  Searching/SearchFolder.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;


namespace Incollector.Searching {
	
	public enum LogicalOperator {
		And = 0,
		Or  = 1
	}
	
	
	public struct SearchFolder {
		
		private int             id;
		private string          name;
		private LogicalOperator searchOperator;
		private ArrayList       queries;
		private bool            exists;
		private bool            special;
		
		
		public int ID {
			get { return id;  }
			set { id = value; }
		}
		
		public string Name {
			get { return name;  }
			set { name = value; }
		}
		
		public LogicalOperator SearchOperator {
			get { return searchOperator;  }
			set { searchOperator = value; }
		}
		
		public ArrayList Queries {
			get { return queries;  }
			set { queries = value; }
		}
		
		public bool Special {
			get { return this.special; }
			set { this.special = value; }
		}
		
		
		public SearchFolder (int id)
		{
			exists = id != -1;
			
			if (id != -1)
				this = SearchFolders.GetFolder (id);
			else {
				this.special = false;
				this.id = -1;
				this.queries = new ArrayList ();
				this.searchOperator = LogicalOperator.And;
				this.name = "";
			}
		}
		
		
		public void Save ()
		{
			if (exists)
				SearchFolders.Replace (this);
			else {
				SearchFolders.LastID++;
				this.id = SearchFolders.LastID;
				exists = true;
				SearchFolders.Add (this);
			}
		}
		
		
		public void Remove ()
		{
			SearchFolders.Remove (this);
		}
		
		
		public bool CheckEntry (Engine.Entry entry)
		{
			if ((queries == null) || (queries.Count == 0))
				return true;
			
			if (searchOperator == LogicalOperator.And) {
				foreach (Query query in queries)
					if (query.CheckQuery (entry) == false)
						return false;
				
				return true;
			} else {
				foreach (Query query in queries)
					if (query.CheckQuery (entry) == true)
						return true;
				
				return false;
			}
		}
		
		
		public void Activate ()
		{
			SearchFolders.Activate (this);
		}
	}
}
