/*
 *  Types/Note.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Xml;
using System.Reflection;
using Incollector.GUI.Widgets;
using Gtk;
using Mono.Unix;


namespace Incollector.DataTypes {

	public class Quote : DataType {
		
		protected static Gtk.Entry    AuthorWidget = new Gtk.Entry ();
		protected static Gtk.TextView QuoteWidget = new Gtk.TextView ();
		
		protected string _DataAuthor = "";
		protected string _DataQuote = "";
		
		protected string DataAuthor {
			get { return this._DataAuthor;  }
			set { this._DataAuthor = value; }
		}
		
		protected string DataQuote {
			get { return this._DataQuote;  }
			set { this._DataQuote = value; }
		}
		
		
		
		public override string Name {
			get { return "quote"; }
		}
		
		public override string Title {
			get { return Catalog.GetString ("Quote"); }
		}
		
		
		public override Gtk.Widget DefaultFocusWidget {
			get { return AuthorWidget; }
		}
		
		public override Gtk.Widget FocusWidget {
			get { return AuthorWidget; }
		}
		
		public override string MenuText {
			get { return Catalog.GetString ("Add a quote"); }
		}
		
		public override Gtk.Widget EditWidget {
			get {
				Gtk.Table tbl = new Gtk.Table (2, 2, false);
				
				Gtk.Label lbl1 = new Gtk.Label (Catalog.GetString ("Author"));
				lbl1.Xalign = 1;
				lbl1.Yalign = (float) 0.50;
				tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				AuthorWidget = new Gtk.Entry ();
				AuthorWidget.Text = this.DataAuthor;
				Incollector.Engine.Autocompletion.Assign (AuthorWidget, "data:" + Name + ":author");
				tbl.Attach (AuthorWidget, 1, 2, 0, 1,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Shrink | AttachOptions.Fill, 3, 3);
				
				Gtk.Label lbl2 = new Gtk.Label (Catalog.GetString ("Quote"));
				lbl2.Xalign = 1;
				lbl2.Yalign = 0;
				tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				QuoteWidget = new Gtk.TextView ();
				QuoteWidget.Buffer.Text = this.DataQuote;
				QuoteWidget.HeightRequest = 80;
				QuoteWidget.AcceptsTab = false;
				QuoteWidget.WrapMode = Gtk.WrapMode.Word;
				QuoteWidget.PixelsAboveLines = 1;
				QuoteWidget.LeftMargin = 3;
				QuoteWidget.RightMargin = 3;
				
				Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
				scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
				scroll.ShadowType = Gtk.ShadowType.In; 
				scroll.Add (QuoteWidget);
				tbl.Attach (scroll, 1, 2, 1, 2,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Expand | AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();
				return tbl;
			}
		}
		
		public override string EditWidgetContent {
			set { this.DataQuote = value; }
		}
		
		public override Gtk.Widget PreviewWidget {
			get {
				Gtk.Table tbl = new Gtk.Table (2, 2, false);
				
				Gtk.Label lbl1 = new Gtk.Label ();
				lbl1.UseMarkup = true;
				lbl1.Markup    = "<b>" + Catalog.GetString ("Author") + "</b>";
				lbl1.Xalign    = 1;
				lbl1.Yalign    = 0;
				tbl.Attach (lbl1, 0, 1, 0, 1, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				PreviewLabel LabelAuthor  = new PreviewLabel (this.DataAuthor);
				LabelAuthor.Xalign     = 0;
				LabelAuthor.Yalign     = 0;
				
				tbl.Attach (LabelAuthor, 1, 2, 0, 1, AttachOptions.Fill, AttachOptions.Fill, 3, 3);
				
				Gtk.Label lbl2 = new Gtk.Label ();
				lbl2.UseMarkup = true;
				lbl2.Markup    = "<b>" + Catalog.GetString ("Quote") + "</b>";
				lbl2.Xalign    = 1;
				lbl2.Yalign    = 0;
				tbl.Attach (lbl2, 0, 1, 1, 2, AttachOptions.Fill,
							AttachOptions.Fill, 3, 3);
				
				PreviewLabel LabelQuote  = new PreviewLabel (this.DataQuote);
				LabelQuote.Wrap       = true;
				LabelQuote.Xalign     = 0;
				LabelQuote.Yalign     = 0;
				tbl.Attach (LabelQuote, 1, 2, 1, 2,
							AttachOptions.Expand | AttachOptions.Fill,
							AttachOptions.Expand | AttachOptions.Fill, 3, 3);
				
				tbl.ShowAll ();
				
				return tbl;
			}
		}
		
		public override string EntryTitle {
			get { return this.DataQuote; }
		}
		
		public override Gdk.Pixbuf Icon16 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "quote-16.png"); }
		}
		
		public override Gdk.Pixbuf Icon32 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "quote-32.png"); }
		}
		
		public override Gdk.Pixbuf Icon48 {
			get { return new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "quote-48.png"); }
		}
		
		
		
		public Quote ()
		{
		}
		
		
		public override DataType New ()
		{
			return new Quote ();
		}
		
		
		public override DataType Clone ()
		{
			Quote qte = new Quote ();
			
			qte._DataAuthor = this.DataAuthor.Clone () as string;
			qte._DataQuote = this.DataQuote.Clone () as string;
			
			return qte;
		}


		public override void ReadXml (XmlTextReader reader)
		{
			this.DataAuthor = reader.GetAttribute ("author");
			this.DataQuote  = reader.ReadString ();
			
			Incollector.Engine.Autocompletion.Add ("data:" + Name + ":author", this.DataAuthor);
		}
		
		
		public override void WriteXml (XmlTextWriter writer)
		{
			writer.WriteStartElement (Name);
				writer.WriteAttributeString ("author", this.DataAuthor);
				writer.WriteString (this.DataQuote);
			writer.WriteEndElement ();
		}
		
		
		public override void AddChangedEvent (EventHandler hdlr)
		{
			AuthorWidget.Changed       += hdlr;
			QuoteWidget.Buffer.Changed += hdlr;
		}
		
		
		public override bool Search (string query)
		{
			return Engine.Text.Search (query, this.DataAuthor) || Engine.Text.Search (query, this.DataQuote);
		}
		
		
		public override bool IsFormFilledIn ()
		{
			return (AuthorWidget.Text != "") && (QuoteWidget.Buffer.Text != "");
		}
		
		
		public override void SynchronizeData ()
		{
			this._DataAuthor = AuthorWidget.Text;
			this._DataQuote = QuoteWidget.Buffer.Text;
		}
	}
}
